@@ -11,10 +11,11 @@ except ImportError:
 from hypothesis import given, example
 from hypothesis.strategies import integers
 import math
+import struct
 
 from tlslite.utils.cryptomath import isPrime, numBits, numBytes, \
         numberToByteArray, MD5, SHA1, secureHash, HMAC_MD5, HMAC_SHA1, \
-        HMAC_SHA256, HMAC_SHA384, HKDF_expand
+        HMAC_SHA256, HMAC_SHA384, HKDF_expand, bytesToNumber
 
 class TestIsPrime(unittest.TestCase):
     def test_with_small_primes(self):
@@ -35,17 +36,43 @@ class TestIsPrime(unittest.TestCase):
         with self.assertRaises(AssertionError):
             for i in range(100):
                 # OEIS A014233
-                self.assertFalse(isPrime(2047))
-                self.assertFalse(isPrime(1373653))
-                self.assertFalse(isPrime(25326001))
-                self.assertFalse(isPrime(3215031751))
-                self.assertFalse(isPrime(2152302898747))
-                self.assertFalse(isPrime(3474749660383))
-                self.assertFalse(isPrime(341550071728321))
-                self.assertFalse(isPrime(341550071728321))
-                self.assertFalse(isPrime(3825123056546413051))
-                self.assertFalse(isPrime(3825123056546413051))
-                self.assertFalse(isPrime(3825123056546413051))
+                self.assertFalse(isPrime(2047))  # base 1
+                self.assertFalse(isPrime(1373653))  # base 2
+                self.assertFalse(isPrime(25326001))  # base 3
+                self.assertFalse(isPrime(3215031751))  # base 4
+                self.assertFalse(isPrime(2152302898747))  # base 5
+                self.assertFalse(isPrime(3474749660383))  # base 6
+                self.assertFalse(isPrime(341550071728321))  # base 7
+                self.assertFalse(isPrime(341550071728321))  # base 8
+                self.assertFalse(isPrime(3825123056546413051))  # base 9
+                self.assertFalse(isPrime(3825123056546413051))  # base 10
+                self.assertFalse(isPrime(3825123056546413051))  # base 11
+                # Zhang (2007)
+                self.assertFalse(isPrime(318665857834031151167461))  # base 12
+                self.assertFalse(isPrime(3317044064679887385961981))  # base 13
+                # base 14
+                self.assertFalse(isPrime(6003094289670105800312596501))
+                # base 15
+                self.assertFalse(isPrime(59276361075595573263446330101))
+                # base 16
+                self.assertFalse(isPrime(564132928021909221014087501701))
+                # base 17
+                self.assertFalse(isPrime(564132928021909221014087501701))
+                # base 18
+                self.assertFalse(isPrime(1543267864443420616877677640751301))
+                # base 19
+                self.assertFalse(isPrime(1543267864443420616877677640751301))
+                # F. Arnault "Constructing Carmichael Numbers Which Are Strong
+                # Pseudoprimes to Several Bases". Journal of Symbolic
+                # Computation. 20 (2): 151-161. doi:10.1006/jsco.1995.1042.
+                # Section 4.4 Large Example (a pseudoprime to all bases up to
+                # 300)
+                p = int("29 674 495 668 685 510 550 154 174 642 905 332 730 "
+                        "771 991 799 853 043 350 995 075 531 276 838 753 171 "
+                        "770 199 594 238 596 428 121 188 033 664 754 218 345 "
+                        "562 493 168 782 883".replace(" ", ""))
+                self.assertTrue(isPrime(p))
+                self.assertFalse(p * (313 * (p - 1) + 1) * (353 * (p - 1) + 1))
 
     def test_with_big_primes(self):
         # NextPrime[2^256]
@@ -82,6 +109,52 @@ class TestNumberToBytesFunctions(unittest.TestCase):
         self.assertEqual(numberToByteArray(0x0a0b0c, 2),
                          bytearray(b'\x0b\x0c'))
 
+    @given(integers(min_value=0, max_value=0xff))
+    @example(0)
+    @example(0xff)
+    def test_small_number(self, number):
+        self.assertEqual(numberToByteArray(number, 1),
+                         bytearray(struct.pack(">B", number)))
+
+    @given(integers(min_value=0, max_value=0xffffffff))
+    @example(0xffffffff)
+    def test_big_number(self, number):
+        self.assertEqual(numberToByteArray(number, 4),
+                         bytearray(struct.pack(">L", number)))
+
+    def test_very_large_number(self):
+        self.assertEqual(numberToByteArray((1<<128)-1),
+                         bytearray(b'\xff'*16))
+
+    @given(integers(min_value=0, max_value=0xff))
+    @example(0)
+    @example(0xff)
+    def test_small_number_little_endian(self, number):
+        self.assertEqual(numberToByteArray(number, 1, endian="little"),
+                         bytearray(struct.pack("<B", number)))
+
+    @given(integers(min_value=0, max_value=0xffffffff))
+    @example(0xffffffff)
+    def test_big_number(self, number):
+        self.assertEqual(numberToByteArray(number, 4, endian="little"),
+                         bytearray(struct.pack("<L", number)))
+
+    def test_very_large_number(self):
+        self.assertEqual(numberToByteArray((1<<128)-1, endian="little"),
+                         bytearray(b'\xff'*16))
+
+    def test_numberToByteArray_with_not_enough_length_little_endian(self):
+        self.assertEqual(numberToByteArray(0x0a0b0c, 2, endian="little"),
+                         bytearray(b'\x0c\x0b'))
+
+    def test_with_large_number_of_bytes_in_little_endian(self):
+        self.assertEqual(numberToByteArray(1, 16, endian="little"),
+                         bytearray(b'\x01' + b'\x00'*15))
+
+    def test_with_bad_endian_type(self):
+        with self.assertRaises(ValueError):
+            numberToByteArray(1, endian="middle")
+
 class TestNumBits(unittest.TestCase):
 
     @staticmethod
@@ -318,3 +391,61 @@ class TestHashMethods(unittest.TestCase):
                                    b'\x64\x3C\xE8\x0E'
                                    b'\x2A\x9A\xC9\x4F'
                                    b'\xA5\x4C\xA4\x9F'))
+
+class TestBytesToNumber(unittest.TestCase):
+    @given(integers(min_value=0, max_value=0xff))
+    @example(0)
+    @example(0xff)
+    def test_small_numbers(self, number):
+        self.assertEqual(bytesToNumber(bytearray(struct.pack(">B", number))),
+                         number)
+
+    @given(integers(min_value=0, max_value=0xffffffff))
+    @example(0xffffffff)
+    def test_multi_byte_numbers(self, number):
+        self.assertEqual(bytesToNumber(bytearray(struct.pack(">I", number))),
+                         number)
+
+    def test_very_long_numbers(self):
+        self.assertEqual(bytesToNumber(bytearray(b'\x00' * 16 + b'\x80')),
+                         0x80)
+        self.assertEqual(bytesToNumber(bytearray(b'\x80' + b'\x00' * 16)),
+                         1<<(8 * 16 + 7))
+        self.assertEqual(bytesToNumber(bytearray(b'\xff'*16)),
+                         (1<<(8*16))-1)
+
+    @given(integers(min_value=0, max_value=0xff))
+    @example(0)
+    @example(0xff)
+    def test_small_numbers_little_endian(self, number):
+        self.assertEqual(bytesToNumber(bytearray(struct.pack("<B", number)),
+                                       "little"),
+                         number)
+
+    @given(integers(min_value=0, max_value=0xffffffff))
+    @example(0xffffffff)
+    def test_multi_byte_numbers_little_endian(self, number):
+        self.assertEqual(bytesToNumber(bytearray(struct.pack("<I", number)),
+                                       "little"),
+                         number)
+
+    def test_very_long_numbers_little_endian(self):
+        self.assertEqual(bytesToNumber(bytearray(b'\x80' + b'\x00' * 16),
+                                       "little"),
+                         0x80)
+        self.assertEqual(bytesToNumber(bytearray(b'\x00'*16 + b'\x80'),
+                                       "little"),
+                         1<<(8 * 16 + 7))
+        self.assertEqual(bytesToNumber(bytearray(b'\xff'*16),
+                                       "little"),
+                         (1<<(8*16))-1)
+
+    def test_with_unknown_type(self):
+        with self.assertRaises(ValueError):
+            bytesToNumber(bytearray(b'\xf0'), "middle")
+
+    def test_with_empty_string(self):
+        self.assertEqual(0, bytesToNumber(b''))
+
+    def test_with_empty_string_little_endian(self):
+        self.assertEqual(0, bytesToNumber(b'', "little"))
